# -*- coding: utf-8 -*-

"""
MEDIA:
    El Heraldo, Honduras

USAGE:
    $ cd heraldoHn/
    ------------------------------------------------------------------------------------------------------------
    ## Get all the news from the most current to the oldest. It's necessary to use the parse_date_files.py file
    for the news contained in noticias.json being splitted into files by date. ##
    
    $ scrapy crawl noticias --nolog -s filename=noticias.json
    ------------------------------------------------------------------------------------------------------------
    ## Get all the news from the most current to a specific date. ##

    scrapy crawl noticias --nolog -s filename=noticias.json -a year=2018 -a month=3 -a day=9
"""

import scrapy, re, json
from datetime import datetime, date
from heraldoHn.items import NoticiasItem


TAG_RE = re.compile(r'<[^>]+>')
def remove_tags(text):
    return TAG_RE.sub('', text)

# LOC = re.compile(r'[A-Z].*?, ?[A-Z].+?\.')


class ImportantFlowData(scrapy.Item):
    """
    Useful data for the flow of the implementation
    """
    to_next_page = scrapy.Field()
    is_last_link = scrapy.Field()
    current_page = scrapy.Field()
    section_url  = scrapy.Field()
    return_url   = scrapy.Field()



class QuotesSpider(scrapy.Spider):
    """
    Basic Scrapy Spider class
    """
    name = "noticias"

    def start_requests(self):
        year = getattr(self, "year", None)
        month = getattr(self, "month", None)
        day = getattr(self, "day", None)

        baseURL = "http://www.elheraldo.hn/"

        sectionList = ["tegucigalpa", "economia", "mundo", "revistas/crimenes",
                       "pais", "sucesos", "deportes", "entretenimiento"]
        # sectionList = ["tegucigalpa"]

        if year is not None and month is not None and day is not None:
            self.stopDate = date(int(year), int(month), int(day))
            for s in sectionList:
                flow_info = ImportantFlowData()
                flow_info['current_page'] = 1
                flow_info['to_next_page'] = False
                
                request = scrapy.Request(url=baseURL + s + "/", callback=self.parse_with_stop_date)
                request.meta['item'] = flow_info
                yield request
        
        else:
            self.stopDate = None
            for s in sectionList:
                flow_info = ImportantFlowData()
                flow_info['current_page'] = 1
                
                request = scrapy.Request(url=baseURL + s + "/", callback=self.parse)
                request.meta['item'] = flow_info
                yield request



    def parse(self, response):
        # print response.url
        searchData = response.meta['item']
        to_next_page = True
        linkSet = set()
        if searchData['current_page'] == 1:
            searchData['section_url'] = response.url
            linkSet = linkSet.union(set(response.xpath('//article[@id="destacada"]/a/@href').extract()))
            linkSet = linkSet.union(set(response.xpath('//aside[@id="mini_sidebar"]/article/a/@href').extract()))
            linkSet = linkSet.union(set(response.xpath('//section[@id="principal"]/article/a/@href').extract()))
            linkSet = linkSet.union(set(response.xpath('//aside[@id="otras_noticias"]/article/a/@href').extract()))
            linkSet = linkSet.union(set(response.xpath('//div[@class="contenedor"]/article/a/@href').extract()))
            linkSet = linkSet.union(set(response.xpath('//article[@class="nobordes"]/div/a/@href').extract()))
            try:
                linkSet.remove(searchData['section_url'])
            except (KeyError, ValueError), e:
                pass

        else:
            linkSet = linkSet.union(set(response.xpath('//div[@class="contenedor"]/article/a/@href').extract()))
            linkSet = linkSet.union(set(response.xpath('//article[@class="nobordes"]/div/a/@href').extract()))
            try:
                linkSet.remove(searchData['section_url'])
            except (KeyError, ValueError), e:
                pass
            
            if len(linkSet) <= 0:
                to_next_page = False

        for link in linkSet:
            yield scrapy.Request(url=link, callback=self.parse_item)

        if to_next_page:
            searchData['current_page'] += 1
            page = searchData['current_page']
            url = searchData['section_url']
            
            request = scrapy.Request(url=url + "?page=" + str(page), callback=self.parse)
            request.meta['item'] = searchData
            yield request



    def parse_with_stop_date(self, response):
        searchData = response.meta['item']
        to_next_page = searchData['to_next_page']

        if not to_next_page:
            if searchData['current_page'] == 1:
                searchData['section_url'] = response.url
                linkList = response.xpath('//article[@id="destacada"]/a/@href').extract()
                linkList.extend(response.xpath('//aside[@id="mini_sidebar"]/article/a/@href').extract())
                linkList.extend(response.xpath('//section[@id="principal"]/article/a/@href').extract())
                linkList.extend(response.xpath('//aside[@id="otras_noticias"]/article/a/@href').extract())
                linkList.extend(response.xpath('//div[@class="contenedor"]/article/a/@href').extract())
                linkList.extend(response.xpath('//article[@class="nobordes"]/div/a/@href').extract())
                try:
                    linkList.remove(searchData['section_url'])
                except (KeyError, ValueError), e:
                    pass

            else:
                linkList = response.xpath('//div[@class="contenedor"]/article/a/@href').extract()
                linkList.extend(response.xpath('//article[@class="nobordes"]/div/a/@href').extract())
                try:
                    linkList.remove(searchData['section_url'])
                except (KeyError, ValueError), e:
                    pass

            newsList = []
            for link in linkList:
                if not link in newsList:
                    newsList.append(link)

            for link in newsList:
                flow_info = ImportantFlowData()
                flow_info['return_url'] = response.url
                flow_info['current_page'] = searchData['current_page']
                flow_info['section_url'] = searchData['section_url']
                
                if link == linkList[-1]: flow_info['is_last_link'] = True
                else: flow_info['is_last_link'] = False
                
                reqst = scrapy.Request(url=link, callback=self.parse_item_with_stop_date)
                reqst.meta['item'] = flow_info
                yield reqst

        else:
            searchData['to_next_page'] = False
            searchData['current_page'] += 1
            page = searchData['current_page']
            url = searchData['section_url']
            
            request = scrapy.Request(url=url + "?page=" + str(page), callback=self.parse_with_stop_date)
            request.meta['item'] = searchData
            yield request



    def parse_item(self, response):
        item = NoticiasItem()
        d = response.xpath('//time/text()').extract_first()
        res = remove_tags(response.xpath('//script[@type="application/ld+json"]').extract_first())
        newsData = json.loads(res)

        news_date = datetime.strptime(d, '%d.%m.%Y').isoformat("T")
        title = newsData['headline']

        try:
            topic = newsData['articleSection']
        except:
            topic = None

        # news_loc = None
        text = newsData['articleBody']
        # if text.find(u'\u00a0') >= 0:
        #     loc = text[:text.find(u'\u00a0')] + "."
        #     m = LOC.match(loc)
        #     if m:
        #         news_loc = m.group(0)
        #         text = text[text.find(u'\u00a0') + 1:]

        
        ## News item info ##
        item['date']  = news_date
        item['title'] = title
        item['topic'] = topic
        # item['location'] = news_loc
        item['text']  = text.strip()
        item['url']   = response.url

        yield item



    def parse_item_with_stop_date(self, response):
        d = response.xpath('//time/text()').extract_first()
        dt = datetime.strptime(d, '%d.%m.%Y').date()

        if dt >= self.stopDate:
            flow_info = response.meta['item']
            item = NoticiasItem()
            res = remove_tags(response.xpath('//script[@type="application/ld+json"]').extract_first())
            newsData = json.loads(res)

            news_date = datetime.strptime(d, '%d.%m.%Y').isoformat("T")
            title = newsData['headline']

            try:
                topic = newsData['articleSection']
            except:
                topic = None

            # news_loc = None
            text = newsData['articleBody']
            # if text.find(u'\u00a0') >= 0:
            #     loc = text[:text.find(u'\u00a0')] + "."
            #     m = LOC.match(loc)
            #     if m:
            #         news_loc = m.group(0)
            #         text = text[text.find(u'\u00a0')+1:]


            ## News item info ##
            item['date']  = news_date
            item['title'] = title
            item['topic'] = topic
            # item['location'] = news_loc
            item['text']  = text.strip()
            item['url']   = response.url

            yield item

            if flow_info['is_last_link']:
                flow_info['to_next_page'] = True
                
                request = scrapy.Request(url=flow_info['return_url'], callback=self.parse_with_stop_date, dont_filter=True)
                request.meta['item'] = flow_info
                yield request
