# -*- coding: utf-8 -*-

"""
MEDIA:
    Puntual, EDOMEX

USAGE:
    ## Get all the news from a specific date. ##
    ---------------------------------------------------------------------------------------------
    $ cd diarioPuntual/
    $ scrapy crawl noticias --nolog -s filename=2018-09-05.json -a year=2018 -a month=9 -a day=5
"""

import scrapy, re
from diarioPuntual.items import NoticiasItem
from datetime import datetime, timedelta, tzinfo


TAG_RE = re.compile(r'<[^>]+>')
def remove_tags(text):
    return TAG_RE.sub('', text)


class UTC(tzinfo):
    """
    Class for Time Zone
    """
    def utcoffset(self, dt):
        ## Time zone for EDOMEX: UTC-6 ##
        return timedelta(hours=-6)

    def tzname(self, dt):
        ## Time zone name ##
        return 'UTC-6'



class QuotesSpider(scrapy.Spider):
    """
    Basic Scrapy Spider class
    """
    name = "noticias"


    def start_requests(self):
        self.tz    = UTC()
        self.year  = getattr(self, "year", None)
        self.month = getattr(self, "month", None)
        self.day   = getattr(self, "day", None)

        baseURL = "http://diario-puntual.com.mx/{0}/{1}/{2}/".format(self.year, self.month.zfill(2), self.day.zfill(2))
        
        yield scrapy.Request(url=baseURL, callback=self.parse)



    def parse(self, response):
        for link in response.css('div.post-column').css('h2.posttitle > a::attr(href)').extract():
            yield scrapy.Request(url=link, callback=self.parse_item)

        next_page = response.css('div.archive-pagination').xpath('./a[@class="next page-numbers"]/@href').extract_first()
        if next_page is not None:
            yield scrapy.Request(url=next_page, callback=self.parse)



    def parse_item(self, response):
        item = NoticiasItem()
        text = ''

        news_date = datetime(int(self.year), int(self.month), int(self.day), tzinfo=self.tz).isoformat("T")

        title = response.css('div.post-container').css('h1.post-title').extract_first()
        if title is not None : title = remove_tags(title)

        topic = None

        for p in response.css('div.post-column > article').css('p').extract():
            p = remove_tags(p)
            text += p + "\n"

        ## News item info ##
        item['date']  = news_date
        item['title'] = title
        item['topic'] = topic
        item['text']  = text.strip()
        item['url']   = response.url

        yield item
