# -*- coding: utf-8 -*-

"""
MEDIA:
    El Comentario, Colima

USAGE:
    ## Get all the news from a specific date. ##
    ---------------------------------------------------------------------------------------------
    $ cd elComentario/
    $ scrapy crawl noticias --nolog -s filename=2018-09-05.json -a year=2018 -a month=9 -a day=5
"""

import scrapy, re
from elComentario.items import NoticiasItem


TAG_RE = re.compile(r'<[^>]+>')
def remove_tags(text):
    return TAG_RE.sub('', text)



class QuotesSpider(scrapy.Spider):
    """
    Basic Scrapy Spider class
    """
    name = "noticias"


    def start_requests(self):
        year  = getattr(self, "year", None)
        month = getattr(self, "month", None)
        day   = getattr(self, "day", None)

        baseURL = "https://elcomentario.ucol.mx/{0}/{1}/{2}/".format(year, month.zfill(2), day.zfill(2))
        
        yield scrapy.Request(url=baseURL, callback=self.parse)



    def parse(self, response):
        for link in response.css('div.articles').xpath('./article/div[@class="cnt"]/h3/a/@href').extract():
            yield scrapy.Request(url=link, callback=self.parse_item)

        next_page = response.css('div.post-pagination').xpath('./a[@title="Next page"]/@href').extract_first()
        if next_page is not None:
            yield scrapy.Request(url=next_page, callback=self.parse)



    def parse_item(self, response):
        item = NoticiasItem()
        text = ''

        news_date = response.xpath('//meta[@property="article:published_time"]/@content').extract_first()

        title = response.xpath('//header/h1').extract_first()
        if title is not None : title = remove_tags(title)

        topic = response.css('a.theme').extract_first()
        if topic is not None : topic = remove_tags(topic)

        for p in response.css('div.pf-content').css('p').extract():
            p = remove_tags(p)
            text += p + "\n"
        text = text.strip()

        ## News item info ##
        item['date']  = news_date
        item['title'] = title
        item['topic'] = topic
        item['text']  = text
        item['url']   = response.url

        yield item
