# -*- coding: utf-8 -*-

"""
MEDIA:
    El Sur, Guerrero

USAGE:
    ## Get all the news from a specific date. ##
    ---------------------------------------------------------------------------------------------
    $ cd elComentario/
    $ scrapy crawl noticias --nolog -s filename=2018-09-05.json -a year=2018 -a month=9 -a day=5
"""

import scrapy, re
from elSur.items import NoticiasItem


TAG_RE = re.compile(r'<[^>]+>')
def remove_tags(text):
    return TAG_RE.sub('', text)

HEAD_RE_1 = re.compile(r'Texto:.*?[/y] Foto:.*?\n(.*?[0-9]{1,2} de [a-zA-Z]*? de [0-9]{4}\.)?')
HEAD_RE_2 = re.compile(r'Texto y foto:.*?\n(.*?[0-9]{1,2} de [a-zA-Z]*? de [0-9]{4}\.)?')


class QuotesSpider(scrapy.Spider):
    """
    Basic Scrapy Spider class
    """
    name = "noticias"


    def start_requests(self):
        year  = getattr(self, "year", None)
        month = getattr(self, "month", None)
        day   = getattr(self, "day", None)

        baseURL = "https://suracapulco.mx/{0}/{1}/{2}/".format(year, month.zfill(2), day.zfill(2))
        
        yield scrapy.Request(url=baseURL, callback=self.parse)



    def parse(self, response):
        for link in response.css('div.dslc-blog-posts').css('div.dslc-blog-post-title > h2 > a::attr(href)').extract():
            yield scrapy.Request(url=link, callback=self.parse_item)

        pag_lst = response.css('div.dslc-pagination > ul > li')
        if len(pag_lst) > 0:
            del pag_lst[0]
            del pag_lst[0]
            next_page = None

            for li_obj in pag_lst:
                li = remove_tags(li_obj.extract())
                if not li.isdigit():
                    next_page = li_obj.xpath('./a/@href').extract_first()
                    break
                
            if next_page is not None : yield scrapy.Request(url=next_page, callback=self.parse)



    def parse_item(self, response):
        item = NoticiasItem()
        text = ''

        news_date = response.xpath('//meta[@property="article:published_time"]/@content').extract_first()

        title = response.css('div.dslc-tp-title > h1').extract_first()
        if title is not None : title = remove_tags(title)

        topic = response.css('div.dslc-tp-meta').xpath('./ul/li[3]/a[1]').extract_first()
        if topic is not None : topic = remove_tags(topic)

        for p in response.xpath('//div[@id="dslc-theme-content-inner"]').css('p').extract():
            p = remove_tags(p)
            text += p + "\n"

        dateline = response.css('span.dateline').extract_first()
        if dateline is not None:
            dateline = remove_tags(dateline)
            text = text.replace(dateline, '')

        text = text.replace(u'\u00a0', ' ')
        text = HEAD_RE_1.sub('', text)
        text = HEAD_RE_2.sub('', text)

        ## News item info ##
        item['date']  = news_date
        item['title'] = title
        item['topic'] = topic
        item['text']  = text.strip()
        item['url']   = response.url

        yield item
