# -*- coding: utf-8 -*-

"""
MEDIA:
    Expreso Chiapas, Chiapas
USAGE:
    ## Get all the news from a specific date. ##
    ---------------------------------------------------------------------------------------------
    $ cd expresoChiapas/
    $ scrapy crawl noticias --nolog -s filename=2018-01-30.json -a year=2018 -a month=1 -a day=30
"""

import scrapy, re, json
from expresoChiapas.items import NoticiasItem
from datetime import datetime, timedelta, tzinfo


TAG_RE = re.compile(r'<[^>]+>')
def remove_tags(text):
    return TAG_RE.sub('', text)

# CLEAN_RE = re.compile(r'\A.*?\sl\s')



class QuotesSpider(scrapy.Spider):
    """
    Basic Scrapy Spider class
    """
    name = "noticias"


    def start_requests(self):
        year = getattr(self, "year", None)
        month = getattr(self, "month", None)
        day = getattr(self, "day", None)

        baseURL = "http://expresochiapas.com/noticias/" + year + "/" + month.zfill(2) + "/" + day.zfill(2)

        yield scrapy.Request(url=baseURL, callback=self.parse)



    def parse(self, response):
        for link in response.css('div.penci-archive__list_posts').css('h2.entry-title > a::attr(href)').extract():
            yield scrapy.Request(url=link, callback=self.parse_item)

        next_page = response.css('div.nav-links').css('a.next::attr(href)').extract_first()
        if next_page is not None:
            yield scrapy.Request(url=next_page, callback=self.parse)



    def parse_item(self, response):
        item = NoticiasItem()
        text = ''

        news_date = response.css('time.entry-date::attr(datetime)').extract_first()

        title = response.css('h1.entry-title').extract_first()
        if title is not None: title = remove_tags(title)

        topic = response.css('div.penci-entry-categories').css('a').extract_first()
        if topic is not None: topic = remove_tags(topic)

        for p in response.css('div.penci-entry-content > p').extract():
            p = p.replace("<br>", "\n")
            text += remove_tags(p) + "\n"


        ## News item info ##
        item['date']  = news_date
        item['title'] = title
        item['topic'] = topic
        item['text']  = text.strip()
        item['url']   = response.url

        yield item
