# -*- coding: utf-8 -*-

"""
MEDIA:
    Diario Co Latino, El Salvador

USAGE:
    ## Get all the news from a specific date. ##
    ---------------------------------------------------------------------------------------------
    $ cd diarioCoLatino/
    $ scrapy crawl noticias --nolog -s filename=2018-02-23.json -a year=2018 -a month=2 -a day=23
"""

import scrapy, re
from diarioCoLatino.items import NoticiasItem


TAG_RE = re.compile(r'<[^>]+>')
def remove_tags(text):
    return TAG_RE.sub('', text)


AUTH_RE = re.compile(r'\nPor.+?\n')
TW_RE = re.compile(r'\n((\| )?Twitter:\s+)?@[\w.%+-]+.\n', re.I)
LOC_RE = re.compile(r'\n.*?\/(PL|AFP|DPA|SIGNIS ALC)\n', re.I)
EM_RE = re.compile(r'\n((Email|Correo electr.{1,3}nico|Comentarios?):\s)?[\w.-]+@[\w-]+(\.[a-zA-Z]{2,6}){1,2}\s?\n')



class QuotesSpider(scrapy.Spider):
    """
    Basic Scrapy Spider class
    """
    name = "noticias"


    def start_requests(self):
        year = getattr(self, "year", None)
        month = getattr(self, "month", None)
        day = getattr(self, "day", None)

        self.baseURL = "https://www.diariocolatino.com/" + year + "/" + month.zfill(2) + "/" + day.zfill(2)

        yield scrapy.Request(url=self.baseURL, callback=self.parse)



    def parse(self, response):
        yield scrapy.Request(url=response.url, callback=self.parse_page, dont_filter=True)

        pagination = response.xpath('//div[@class="pagination"]/a/@href').extract()
        if len(pagination) > 0:
            pagination = pagination[-1].strip('/')
            pages = int(pagination[pagination.rfind('/') + 1:])

            for page in range(1, pages):
                yield scrapy.Request(url=self.baseURL + "/page/" + str(page + 1), callback=self.parse_page)



    def parse_page(self, response):
        for link in response.css('div.content').css('div.post-listing').xpath('./article/h2/a/@href').extract():
            yield scrapy.Request(url=link, callback=self.parse_item)



    def parse_item(self, response):
        item = NoticiasItem()
        text = ''

        # La fecha obtenida ya incluye formato y zona horaria
        news_date = response.xpath('//meta[@property="article:published_time"]/@content').extract_first()
        news_title = remove_tags(response.css('h1.entry-title').css('span').extract_first()).strip()

        news_topic = None

        for p in response.xpath('//div[@class="entry"]/p').extract():
            text += remove_tags(p) + "\n"

        if text == '':
            for p in response.xpath('//div[@class="entry"]/div/span').extract():
                text += remove_tags(p) + "\n"

        text = "\n" + text
        """ Obtiene autor """
        news_author = None
        res = AUTH_RE.match(text)
        if res:
            m = res.group(0)
            news_author = m[m.find('Por')+len('Por'):].strip()
            text = text[text.find(m) + len(m):].strip()
            text = "\n" + text

        """ Elimina twitter """
        news_twitter = None
        res = TW_RE.search(text)
        if res:
            m = res.group(0)
            news_twitter = m.strip()
            text = text[text.find(m) + len(m):].strip()
            text = "\n" + text

        """ Obtiene lugar """
        news_loc = None
        res = LOC_RE.match(text)
        if res:
            m = res.group(0)
            if m[m.find('/') + 1:].strip().lower() != 'dpa':
                news_loc = m[:m.find('/')].strip()
                text = text[text.find(m) + len(m):].strip()
                text = "\n" + text
            else:
                text = text[text.find(m) + len(m):].strip()
                text = "\n" + text

        """ Elimina correo """
        news_email = None
        res = EM_RE.search(text)
        if res:
            m = res.group(0)
            news_email = m.strip()
            # text = text[text.find(m) + len(m):].strip()
            text = text.replace(m, '').strip()
            text = "\n" + text

        res = EM_RE.search(text)
        if res:
            m = res.group(0)
            news_email = m.strip()
            # text = text[text.find(m) + len(m):].strip()
            text = text.replace(m, '').strip()
            text = "\n" + text

        text = text.replace("\n@Diario Co Latino\n", '').strip()
        text = "\n" + text
        text = text.replace("\nDiario Co Latino\n", '').strip()
        text = "\n" + text
        text = text.replace("\nCo Latino\n", '').strip()


        ## News item info ##
        item['date']   = news_date
        item['title']  = news_title
        item['topic']  = news_topic
        item['author'] = news_author
        item['twitter'] = news_twitter
        item['location'] = news_loc
        item['email'] = news_email
        item['text']  = text.strip()
        item['url']   = response.url

        yield item
