# -*- coding: utf-8 -*-

"""
MEDIA:
    La Jornada, CDMX
USAGE:
    ## Get all the news from a specific date. ##
    ---------------------------------------------------------------------------------------------
    $ cd laJornada/
    $ scrapy crawl noticias --nolog -s filename=2017-03-22.json -a year=2017 -a month=3 -a day=22
"""

import scrapy, re, json
from laJornada.items import NoticiasItem
from datetime import datetime, timedelta, tzinfo


TAG_RE = re.compile(r'<[^>]+>')
def remove_tags(text):
    return TAG_RE.sub('', text)



class QuotesSpider(scrapy.Spider):
    """
    Basic Scrapy Spider class
    """
    name = "noticias"


    def start_requests(self):
        year = getattr(self, "year", None)
        month = getattr(self, "month", None)
        self.day = getattr(self, "day", None)

        self.this_date = year + "-" + month.zfill(2) + "-" + self.day.zfill(2)

        self.baseURL = "https://www.jornada.com.mx/" + year + "/" + month.zfill(2) + "/" + self.day.zfill(2) + "/"

        section_list = ["politica", "mundo",    "capital", "cultura", "deportes",
                        "economia", "sociedad", "estados", "espectaculos"]

        for s in section_list:
            yield scrapy.Request(url=self.baseURL + s, callback=self.parse)



    def parse(self, response):
        link_set = set(response.css('div.section-cont').css('a.cabeza::attr(href)').extract())
        
        for link in link_set:
            yield scrapy.Request(url=self.baseURL + link, callback=self.parse_item)



    def parse_item(self, response):
        item = NoticiasItem()
        text = ''

        news_date = response.xpath('//meta[@property="article:published_time"]/@content').extract_first()
        d, t = news_date.split("T")
        if d != self.this_date:
            news_date = d[:d.rfind('-')] + "-" + self.day.zfill(2) + "T" + t

        title = response.css('div.cabeza').extract_first()
        if title is not None: title = remove_tags(title)

        topic = response.css('img.title::attr(title)').extract_first()
        if topic is not None: topic = remove_tags(topic)

        for p in response.css('div.text').css('p').extract():
            p = p.replace("<br>", "\n")
            text += remove_tags(p) + "\n"


        ## News item info ##
        item['date']  = news_date
        item['title'] = title
        item['topic'] = topic
        item['text']  = text.strip()
        item['url']   = response.url

        yield item
