# -*- coding: utf-8 -*-

"""
MEDIA:
    La Jornada de Oriente, Puebla

USAGE:
    ## Get all the news from a specific date. ##
    ---------------------------------------------------------------------------------------------
    $ cd laJornadaOte/
    $ scrapy crawl noticias --nolog -s filename=2017-03-22.json -a year=2017 -a month=3 -a day=22
"""

import scrapy, re
from laJornadaOte.items import NoticiasItem


TAG_RE = re.compile(r'<[^>]+>')
def remove_tags(text):
    return TAG_RE.sub('', text)


class QuotesSpider(scrapy.Spider):
    """
    Basic Scrapy Spider class
    """
    name = "noticias"

    def start_requests(self):
        year = getattr(self, 'year', None)
        month = getattr(self, 'month', None)
        day = getattr(self, 'day', None)

        self.baseURL='http://www.lajornadadeoriente.com.mx/'+year+'/'+month+'/'+day

        yield scrapy.Request(url=self.baseURL, callback=self.parse)



    def parse(self, response):
        for link in response.css('div.content').css('h3.entry-title > a::attr(href)').extract():
            yield scrapy.Request(url=link, callback=self.parse_item)

        next_page = response.css('ul.page-numbers').css('a.next::attr(href)').extract_first()
        if next_page is not None:
            yield scrapy.Request(url=next_page, callback=self.parse)



    def parse_item(self, response):
        item = NoticiasItem()
        text = ''

        ## news date already includes time zone ##
        news_date = response.xpath('//meta[@property="article:published_time"]/@content').extract_first()

        title = response.css('h2.entry-title').extract_first()
        if title is not None : title = remove_tags(title)

        topic = response.xpath('//a[@rel="category tag"]').extract_first()
        if topic is not None : topic = remove_tags(topic)

        for p in response.css('div.entry-content > p').extract():
            text += remove_tags(p) + '\n'
        
        
        ## News item info ##
        item['date']  = news_date
        item['title'] = title
        item['topic'] = topic
        item['text']  = text.strip()
        item['url']   = response.url

        yield item

