# -*- coding: utf-8 -*-

"""
MEDIA:
    Lector MX, Yucatán

USAGE:
    ## Get all the news from a specific date. ##
    ---------------------------------------------------------------------------------------------
    $ cd lectorMX/
    $ scrapy crawl noticias --nolog -s filename=2017-03-30.json -a year=2017 -a month=3 -a day=30
"""

import scrapy, re
from lectorMX.items import NoticiasItem
from datetime import datetime, timedelta, tzinfo


TAG_RE = re.compile(r'<[^>]+>')
def remove_tags(text):
    return TAG_RE.sub('', text)


class UTC(tzinfo):
    """
    Class for Time Zone
    """
    def utcoffset(self, dt):
        ## Time zone for Yucatán: UTC-6 ##
        return timedelta(hours=-6)

    def tzname(self, dt):
        ## Time zone name ##
        return 'UTC-6'



class QuotesSpider(scrapy.Spider):
    """
    Basic Scrapy Spider class
    """
    name = "noticias"

    def start_requests(self):
        tz = UTC()
        year = getattr(self, 'year', None)
        month = getattr(self, 'month', None)
        day = getattr(self, 'day', None)

        self.news_date = datetime(int(year), int(month), int(day), tzinfo=tz).isoformat('T')
        
        baseURL = 'http://lectormx.com/' + year + '/' + month + '/' + day

        yield scrapy.Request(url=baseURL, callback=self.parse)



    def parse(self, response):
        for link in response.css('div.paginated_content').css('h2.entry-title > a::attr(href)').extract():
            yield scrapy.Request(url=link, callback=self.parse_item)

        next_page = response.css('div.archive-pagination').css('a.next::attr(href)').extract_first()
        if next_page is not None:
            yield scrapy.Request(url=next_page, callback=self.parse)



    def parse_item(self, response):
        text = ''
        item = NoticiasItem()

        title = response.css('h1.entry-title').extract_first()
        if title is not None: title = remove_tags(title)

        topic = response.xpath('//a[@rel="tag"]').extract_first()
        if topic is not None: topic = remove_tags(topic)

        for p in response.css('div.entry-content > p').extract():
            text += remove_tags(p) + '\n'


        ## News item info ##
        item['date']  = self.news_date
        item['title'] = title
        item['topic'] = topic
        item['text']  = text.strip()
        item['url']   = response.url
        
        yield item

