# -*- coding: utf-8 -*-

"""
MEDIA:
    Periódico Correo, Guanajuato

USAGE:
    ## Get all the news from a specific date. ##
    ---------------------------------------------------------------------------------------------
    $ cd periodicoCorreo/
    $ scrapy crawl noticias --nolog -s filename=2018-01-29.json -a year=2018 -a month=1 -a day=29
"""

import scrapy, re
from periodicoCorreo.items import NoticiasItem


TAG_RE = re.compile(r'<[^>]+>')
def remove_tags(text):
    return TAG_RE.sub('', text)


""" Regular expression for parsing some data """
# LOC_RE = re.compile(r'\n.+?,? ?.+? ?\. ?- ?')
# G_RE = re.compile(r' ?- ?')
# EM_RE = re.compile(r'((Email|Correo electr.{1,3}nico|Comentarios?):\s)?[\w.-]+@[\w-]+(\.[a-zA-Z]{2,6}){1,2}\s?')
# TW_RE = re.compile(r'M.{1,3}s de la P.{1,3}lvora en Twitter: @[\w.%+-]+.', re.I)
# TW2_RE = re.compile(r'((\| )?Twitter:\s+)?(@[\w.%+-]+.)?', re.I)
# TAG2_RE = re.compile(r'\ntransition_[^\]]+\]')
# TAG3_RE = re.compile(r'\[[^\]]+[\]\n]')


class QuotesSpider(scrapy.Spider):
    """
    Basic Scrapy Spider class
    """
    name = "noticias"

    def start_requests(self):
        year = getattr(self, "year", None)
        month = getattr(self, "month", None)
        day = getattr(self, "day", None)

        baseURL = "https://periodicocorreo.com.mx/" + year + "/" + month.zfill(2) + "/" + day.zfill(2)

        yield scrapy.Request(url=baseURL, callback=self.parse)



    def parse(self, response):
        for link in response.css('div.main-content').css('h2.card-title > a::attr(href)').extract():
            yield scrapy.Request(url=link, callback=self.parse_item)

        next_page = response.css('div.archive-navigation').css('a.next::attr(href)').extract_first()
        if next_page is not None:
            yield scrapy.Request(url=next_page, callback=self.parse)



    def parse_item(self, response):
        item = NoticiasItem()
        text = ''

        ## news date already includes time zone ##
        news_date = response.css('div.entry-meta').css('time.entry-date::attr(datetime)').extract_first()
        
        title = response.css('h1.entry-title').extract_first()
        if title is not None: title = remove_tags(title)

        topic = response.css('ul.post-categories').css('a').extract_first()
        if topic is not None: topic = remove_tags(topic)

        entry_content = response.css('div.entry-content > article > p').extract()
        if len(entry_content) <= 0: entry_content = response.css('div.pane-content').css('p').extract()
        if len(entry_content) <= 0: entry_content = response.css('div.post-content').css('p').extract()
        for p in entry_content:
            text += remove_tags(p) + "\n"


        ## News item info ##
        item['date']  = news_date
        item['title'] = title
        item['topic'] = topic
        item['text']  = text.strip()
        item['url']   = response.url

        yield item

        ## deprecated section ##
        # result = LOC_RE.search(text)
        # if result:
        #     m = result.group(0)
        #     location = G_RE.sub('', m).strip()
        #     if len(location) <= 35:
        #         news_loc = location
        #         text = text[text.find(m)+len(m):]

        # text = EM_RE.sub('', text)
        # text = TW_RE.sub('', text)
        # text = TW2_RE.sub('', text)
        # text = TAG2_RE.sub("\n", text)
        # text = TAG3_RE.sub('', text)
