# -*- coding: utf-8 -*-

"""
MEDIA:
    Periódico Victoria, Durango

USAGE:
    ## Get all the news from a specific date. ##
    ---------------------------------------------------------------------------------------------
    $ cd periodicoVictoria/
    $ scrapy crawl noticias --nolog -s filename=2018-01-28.json -a year=2018 -a month=1 -a day=28
"""

import scrapy, re
from periodicoVictoria.items import NoticiasItem


TAG_RE = re.compile(r'<[^>]+>')
def remove_tags(text):
    return TAG_RE.sub('', text)


""" Regular expression for parsing some data """
# LOC_RE = re.compile(r'\A.+?,.+?(\d{1,2}[\s-][a-zA-Z]+[\s-]\d{4})?\s?\.\s?-\s?', re.S)
# G_RE = re.compile(r'\s?-\s?')
# GT_RE = re.compile(r'&gt;\s?[\w.%+-]+.+')
# EM_RE = re.compile(r'((Email|Correo electr.{1,3}nico|Comentarios?):\s)?[\w.-]+@[\w-]+(\.[a-zA-Z]{2,6}){1,2}\s?')
# TW_RE = re.compile(r'Twitter:\s+[\w.%+-]+.')
# TW2_RE = re.compile(r'(Twitter:\s+)?(@[\w.%+-]+.)?')
# FB_RE = re.compile(r'(Facebook|Blog|Por|Informes|Texto y (F|f)otos):\s+[\w.%+-]+.+')
# URL_RE = re.compile(r'\(?(https?:\/\/)?([w{3}.])?[\w%+-]+(\.[a-zA-Z]{2,6}){1,2}[/\w.#$%&+-]*\)?.')


class QuotesSpider(scrapy.Spider):
    """
    Basic Scrapy Spider class
    """
    name = "noticias"


    def start_requests(self):
        year = getattr(self, "year", None)
        month = getattr(self, "month", None)
        day = getattr(self, "day", None)

        baseURL = "http://periodicovictoria.mx/" + year + "/" + month.zfill(2) + "/" + day.zfill(2)

        yield scrapy.Request(url=baseURL, callback=self.parse)



    def parse(self, response):
        for link in response.css('main.site-main').css('h3.entry-title > a::attr(href)').extract():
            yield scrapy.Request(url=link, callback=self.parse_item)

        next_page = response.css('div.pagination').css('li.go-next > a::attr(href)').extract_first()
        if next_page is not None:
            yield scrapy.Request(url=next_page, callback=self.parse)



    def parse_item(self, response):
        item = NoticiasItem()
        text = ''

        ## news date already includes time zone ##
        news_date = response.xpath('//meta[@property="article:published_time"]/@content').extract_first()
        
        title = response.css('div.entry-wrapper').css('h1.entry-title').extract_first()
        if title is not None: title = remove_tags(title)

        topic = response.xpath('//a[@rel="category tag"]').extract_first()
        if topic is not None: topic = remove_tags(topic)

        author = response.xpath('//a[@rel="author"]').extract_first()
        if author is not None: author = remove_tags(author)

        for p in response.css('div.entry-content').css('p').extract():
            p.replace("<br>", "\n")
            text += remove_tags(p) + "\n"


        ## News item info ##
        item['date']  = news_date
        item['title'] = title
        item['topic'] = topic
        item['author'] = author
        item['text']  = text.strip()
        item['url']   = response.url

        yield item

        ## deprecated section ##
        # for p in response.xpath('//div[@class="post-entry clearfix"]/p').extract():
        #     p = remove_tags(p)
        #     p = p.lstrip().replace(u'\u2013', "-")
        #     p = p.lstrip().replace(u'\u2014', "-")
        #     p = p.lstrip().replace(u'\u00a0', '')
        #     result = LOC_RE.match(p)
        #     if result:
        #         location = G_RE.sub('', result.group(0)).strip()
        #         if location.find("Corresponsal") > -1:
        #             location = location[location.find("Corresponsal") + len("Corresponsal"):].strip()
        #         if len(location) <= 30:
        #             item['location'] = location
        #             p = p.replace(result.group(0), '')
        #             text = ''

        #     text += p + "\n"

        # text = GT_RE.sub('', text)
        # text = EM_RE.sub('', text)
        # text = TW_RE.sub('', text)
        # text = TW2_RE.sub('', text)
        # text = FB_RE.sub('', text)
        # text = URL_RE.sub('', text)
        # if text.find("Corresponsal") > -1:
        #     text = text[text.find("Corresponsal") + len("Corresponsal"):]
