# -*- coding: utf-8 -*-

"""
MEDIA:
    Por Esto!, Yucatán
USAGE:
    ## Get all the news from a specific date. ##
    ---------------------------------------------------------------------------------------------
    $ cd porEsto/
    $ scrapy crawl noticias --nolog -s filename=2018-08-22.json -a year=2018 -a month=8 -a day=22
"""

import scrapy, re, json
from porEsto.items import NoticiasItem
from datetime import datetime, timedelta, tzinfo


TAG_RE = re.compile(r'<[^>]+>')
def remove_tags(text):
    return TAG_RE.sub('', text)



class QuotesSpider(scrapy.Spider):
    """
    Basic Scrapy Spider class
    """
    name = "noticias"


    def start_requests(self):
        year = getattr(self, "year", None)
        month = getattr(self, "month", None)
        day = getattr(self, "day", None)

        baseURL = "http://www.poresto.net/" + year + "/" + month.zfill(2) + "/" + day.zfill(2) + "/"

        yield scrapy.Request(url=baseURL, callback=self.parse)



    def parse(self, response):
        for link in response.css('div.jeg_inner_content').css('h3.jeg_post_title > a::attr(href)').extract():
            yield scrapy.Request(url=link, callback=self.parse_item)

        next_page = response.css('div.jeg_navigation').css('a.next::attr(href)').extract_first()
        if next_page is not None:
            yield scrapy.Request(url=next_page, callback=self.parse)



    def parse_item(self, response):
        item = NoticiasItem()
        text = ''

        news_date = response.xpath('//meta[@property="article:published_time"]/@content').extract_first()

        title = response.css('h1.jeg_post_title').extract_first()
        if title is not None: title = remove_tags(title)

        topic = response.css('div.jeg_meta_category > span > a').extract_first()
        if topic is not None: topic = remove_tags(topic)

        for p in response.css('div.entry-content').css('p').extract():
            text += remove_tags(p) + "\n"

        if text == '':
            text = response.css('div.entry-content').css('div.content-inner').extract_first()
            text.replace("<br>", '')
            text = remove_tags(text)


        ## News item info ##
        item['date']  = news_date
        item['title'] = title
        item['topic'] = topic
        item['text']  = text.strip()
        item['url']   = response.url

        yield item
