# -*- coding: utf-8 -*-

"""
MEDIA:
    Tribuna de los Cabos, Baja California Sur

USAGE:
    ## Get all the news from a specific date. ##
    ---------------------------------------------------------------------------------------------
    $ cd tribunaCabos/
    $ scrapy crawl noticias -t json --nolog -o noticias.json -a year=2017 -a month=3 -a day=5
"""

import scrapy, re
from tribunaCabos.items import NoticiasItem
from datetime import datetime, timedelta, tzinfo


TAG_RE = re.compile(r'<[^>]+>')
def remove_tags(text):
    return TAG_RE.sub('', text)

""" Regular expression for parsing some data """
# # LOC_RE = re.compile(ur'.+,\s+\d?\d\s+[a-zA-Z]+\.?\s\([^\)]+\)\s?\.-?\u2013?\s?', re.UNICODE)
# LOC_RE = re.compile(ur'.+?(\d?\d[\s-][a-zA-Z]+)?\s?(\([^\)]+\))?\s?\.[\u2013-]\s?', re.UNICODE)
# DAT_RE = re.compile(ur',?\s?(\d?\d[\s-][a-zA-Z]+)?\s?(\([^\)]+\))?\s?\.[\u2013-]\s?', re.UNICODE)
# DAT2_RE = re.compile(r',?\sa\s\d{1,2}\sde\s[a-zA-Z]+\sde\s\d{4}')


class UTC(tzinfo):
    """
    Class for Time Zone
    """
    def utcoffset(self, dt):
        ## Time zone for Baja California Sur: UTC-6 ##
        return timedelta(hours=-6)

    def tzname(self, dt):
        ## Time zone name ##
        return 'UTC-6'



class QuotesSpider(scrapy.Spider):
    """
    Basic Scrapy Spider class
    """
    name = "noticias"


    def start_requests(self):
        tz = UTC()
        year = getattr(self, "year", None)
        month = getattr(self, "month", None)
        day = getattr(self, "day", None)

        self.news_date = datetime(int(year), int(month), int(day), tzinfo=tz).isoformat('T')

        baseURL = "http://www.tribunadeloscabos.com.mx/" + year + "/" + month + "/" + day

        yield scrapy.Request(url=baseURL, callback=self.parse)



    def parse(self, response):
        for link in response.css('div.blog-posts').css('h2.entry-title > a::attr(href)').extract():
            yield scrapy.Request(url=link, callback=self.parse_item)

        page_list = response.css('div.pagination').css('li > a::attr(href)').extract()
        if len(page_list) > 0:
            next_page = page_list[-1]
            current_page = response.css('div.pagination').css('li.active > a::attr(href)').extract_first()
            
            if next_page != current_page:
                yield scrapy.Request(url=next_page, callback=self.parse)



    def parse_item(self, response):
        item = NoticiasItem()
        text = ''
        
        title = response.css('div.post-title > h1').extract_first()
        if title is not None: title = remove_tags(title)

        topic = response.css('div.post-category > a').extract_first()
        if topic is not None: topic = remove_tags(topic)

        for p in response.css('div.post-content').css('p').extract():
            text += remove_tags(p) + "\n"


        ## News item info ##
        item['date']  = self.news_date
        item['title'] = title
        item['topic'] = topic
        item['text']  = text
        item['url']   = response.url

        yield item


        ## deprecated section ##
        # author = response.xpath('//span[@class="author vcard"]/a/text()').extract_first()
        # if author is not None and author != '':
        #     item['author'] = author

        # bodyText = response.xpath('//*[@class="entry-content clearfix"]/p').extract()
        # for i in range(0, len(bodyText)):
        #     p = remove_tags(bodyText[i])
        #     if i <= 1:
        #         p = p.lstrip()
        #         result = LOC_RE.match(p)
        #         if result:
        #             loc = DAT_RE.sub('', result.group(0))
        #             item['location'] = DAT2_RE.sub('', loc)
        #             p = LOC_RE.sub('', p)

        #     text += p + "\n"


