# -*- coding: utf-8 -*-

"""
MEDIA:
    Proceso, CDMX

USAGE:
    ## For this crawler 'scrapy-splash' is used because the content is loaded through javascript. ##
    ## Read especs_sitio_proceso.txt file. ##
    
    $ cd proceso/
    ------------------------------------------------------------------------------------------------------------
    ## Get all the news from the most current to the oldest. It's necessary to use the parse_date_files.py file
    for the news contained in noticias.json being splitted into files by date. ##
    
    $ scrapy crawl noticias --nolog -s filename=noticias.json
    ------------------------------------------------------------------------------------------------------------
    ## Get all the news from the most current to a specific year-month date. ##
    
    $ scrapy crawl noticias --nolog -s filename=2018-09.json -a year=2018 -a month=9
"""

import scrapy, re, cfscrape
from proceso.items import NoticiasItem
from datetime import datetime, date, timedelta, tzinfo
from scrapy.http.cookies import CookieJar
from scrapy_splash import SplashRequest, SplashFormRequest


TAG_RE = re.compile(r'<[^>]+>')
def remove_tags(text):
    return TAG_RE.sub('', text)



class UTC(tzinfo):
    """
    Class for Time Zone
    """
    def utcoffset(self, dt):
        ## Time zone for CDMX: UTC-6 ##
        return timedelta(hours=-6)

    def tzname(self, dt):
        ## Time zone name ##
        return 'UTC-6'

USER_AGENT = 'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/67.0.3396.87 Safari/537.36'

script = """
function main(splash)
  splash:init_cookies(splash.args.cookies)
  assert(splash:go{
    splash.args.url,
    headers=splash.args.headers,
    http_method=splash.args.http_method,
    body=splash.args.body,
    })
  assert(splash:wait(0.5))

  local entries = splash:history()
  local last_response = entries[#entries].response
  return {
    url = splash:url(),
    headers = last_response.headers,
    http_status = last_response.status,
    cookies = splash:get_cookies(),
    html = splash:html(),
  }
end
"""

class QuotesSpider(scrapy.Spider):
    """
    Basic Scrapy Spider class
    """
    name = "noticias"
    
    def start_requests(self):
        self.tz = UTC()
        year = getattr(self, 'year', None)
        month = getattr(self, 'month', None)

        self.month_parser = {
            'enero'      : '01', 'febrero' : '02',  'marzo'     : '03',  'abril'     : '04',
            'mayo'       : '05', 'junio'   : '06',  'julio'     : '07',  'agosto'    : '08',
            'septiembre' : '09', 'octubre' : '10',  'noviembre' : '11',  'diciembre' : '12'
        }

        self.baseURL = "https://hemeroteca.proceso.com.mx/"
        login_url = "https://hemeroteca.proceso.com.mx/wp-login.php"

        if year is not None and month is not None:
            self.stop_date = date(int(year), int(month), 15)
            # yield scrapy.Request(url=login_url, callback=self.parse_with_stop_date)
        
        else:
            self.stop_date = None
            # yield scrapy.Request(url=login_url, callback=self.parse)

        token, agent = cfscrape.get_tokens(login_url, user_agent=USER_AGENT)
        print token
        print "\n"
        yield SplashRequest(url=login_url, callback=self.parse_login, endpoint='render.html', args={ 'wait': 0.5 }, cookies=token,
                    headers={'User-Agent' : agent})


    def parse_login(self, response):
        return SplashFormRequest.from_response(
            response,
            formdata = {
                'log' : 'carlos_silvaforne@yahoo.com.mx',
                'pwd' : 'carlos_silvaforne@'
            },
            callback = self.after_login,
            dont_click = True
        )



    def after_login(self, response):
        ## Check login succeed before going on ##
        print response.headers
        print "\n"
        print response.real_url
        print "\n"
        print response.request.headers
        print "\n"
        cookie_list = response.request.headers.getlist('Cookie')
        cfc, cfd = cookie_list[0].split(';')
        cfc = cfc.strip().split('=')
        cfd = cfd.strip().split('=')
        cookies = [cfc[1], cfd[1]]
        cookies = {cfc[0]: cfc[1], cfd[0]: cfd[1]}
        
        session_legend = response.css('div.topnav > a').extract()[-1]
        print response.css('h1.entry-title').extract_first()
        print "\n"

        if session_legend is not None:
            session_legend = remove_tags(session_legend)

            if not "Cerrar" in session_legend:
                print "Login failed."

            else:
                print session_legend
                print "\n"
                token, agent = cfscrape.get_tokens(self.baseURL, user_agent=USER_AGENT)
                print token
                print "\n"

                if self.stop_date is None:
                    yield scrapy.Request(url=self.baseURL, callback=self.parse)

                else:
                    self.meta = response.request.meta
                    yield SplashRequest(url=self.baseURL, callback=self.parse_with_stop_date,
                        meta=self.meta,
                        endpoint='execute',
                        cache_args=['lua_source'],
                        args={'lua_source': script},
                        headers={'User-Agent': USER_AGENT}
                    )
                    # request = SplashRequest(url=self.baseURL, callback=self.parse_with_stop_date,
                    #     endpoint='execute',
                    #     cache_args=['lua_source'],
                    #     args={'lua_source': script},
                    #     headers={'User-Agent': USER_AGENT}
                    # )
                    # request.meta['splash']['session_id'] = cookie_list[0]
                    # yield request



        # if "authentication failed" in response.body:
        #     self.logger.error("Login failed.")
        #     return

        # else:
        #     # token, agent = cfscrape.get_tokens(self.baseURL, user_agent=USER_AGENT)
        #     if self.stop_date is None:
        #         yield scrapy.Request(url=self.baseURL, callback=self.parse, dont_filter=True)

        #     else:
        #         yield scrapy.Request(url=self.baseURL, callback=self.parse_with_stop_date, dont_filter=True)
                # yield scrapy.Request(
                #     url=self.baseURL,
                #     callback=self.parse_with_stop_date,
                #     cookies=token,
                #     headers={'User-Agent' : agent}
                # )



    def parse_with_stop_date(self, response):
        print "parse_with_stop_date"
        print "\n"
        print response.css('h1.entry-title').extract_first()
        # print "\n"
        # print response.cookiejar
        print "\n"
        print response.headers
        print "\n"
        
        # session_legend = response.css('div.topnav > a').extract()[-1]
        # if session_legend is not None :
        #     print remove_tags(session_legend)
        #     print "\n"
        # else :
        #     print "No log in."
        
        TO_NEXT_PAGE = True

        for item in response.css('div.catpor-box > div'):
            item_date = item.css('span.catpor-published').extract_first()

            if item_date is not None:
                item_date    = remove_tags(item_date).replace(",", '')
                item_date    = item_date.split(' ')
                item_date[1] = self.month_parser[item_date[1]]
                item_date    = map(int, item_date)
                item_date    = date(item_date[2], item_date[1], item_date[0])
                
                if item_date >= self.stop_date:
                    item_link = item.css('span.catpor-title > a::attr(href)').extract_first()
                    print item_link
                    # token, agent = cfscrape.get_tokens(self.baseURL, user_agent=USER_AGENT)
                    # yield scrapy.Request(url=item_link, callback=self.parse_links, cookies=cookies)
                    # yield scrapy.Request(url=item_link, callback=self.parse_links, cookies=token,
                    #                         headers={'User-Agent' : agent})
                    yield SplashRequest(url=item_link, callback=self.parse_links,
                        endpoint='execute',
                        cache_args=['lua_source'],
                        args={'lua_source': script},
                        headers={'User-Agent': USER_AGENT}
                    )

                else:
                    TO_NEXT_PAGE = False
                    break

        if TO_NEXT_PAGE:
            next_page = response.css('div.page-navigation > div.nav-next > a::attr(href)').extract_first()

            if next_page is not None:
                # yield scrapy.Request(url=next_page, callback=self.parse_with_stop_date)
                yield SplashRequest(url=next_page, callback=self.parse_with_stop_date,
                    endpoint='execute',
                    cache_args=['lua_source'],
                    args={'lua_source': script},
                    headers={'User-Agent': USER_AGENT}
                )



    def parse_links(self, response):
        print "\n\n"
        print response.headers
        for link in response.css('div.post-container > h2 > a::attr(href)').extract():
            # print link
            # token, agent = cfscrape.get_tokens(self.baseURL, user_agent=USER_AGENT)

            # yield SplashRequest(url=link, callback=self.parse_item, endpoint='render.html', args={ 'wait': 0.5 }, cookies=token,
            #         headers={'User-Agent' : agent})
            yield SplashRequest(url=link, callback=self.parse_item,
                endpoint='execute',
                cache_args=['lua_source'],
                args={'lua_source': script},
                headers={'User-Agent': USER_AGENT}
            )



    def parse_item(self, response):
        # if response.url == "https://hemeroteca.proceso.com.mx/?page_id=278958&a51dc26366d99bb5fa29cea4747565fec=420203":
        #     print response.body
        item = NoticiasItem()
        text = ''
        
        news_date = response.xpath('//div[@id="primary"]').css('span.published').extract_first()
        if news_date is not None:
            news_date = remove_tags(news_date)
            print news_date
            d, t = news_date.split(' ')
            d = map(int, d.split("-"))
            t = map(int, t.split(":"))
            news_date = datetime(d[0], d[1], d[2], t[0], t[1], t[2], tzinfo=self.tz).isoformat('T')

        title = response.xpath('//div[@id="primary"]/div/h1').extract_first()
        if title is not None : title = remove_tags(title)

        topic = response.css('span.entry-categories').extract_first()
        if topic is not None : topic = remove_tags(topic)
        
        for paragraph in response.xpath('//div[@id="primary"]').css('div.entry-content > div').css('p').extract():
            text += remove_tags(paragraph) + '\n'
        
        ## News item info ##
        item['date']  = news_date
        item['title'] = title
        item['topic'] = topic
        item['text']  = text.strip()
        item['url']   = response.url
        
        yield item

