# -*- coding: utf-8 -*-

"""
MEDIA:
    Proceso, CDMX

USAGE:
    ## For this crawler 'scrapy-splash' is used because the content is loaded through javascript. ##
    ## Read especs_sitio_proceso.txt file. ##
    
    $ cd proceso/
    ------------------------------------------------------------------------------------------------------------
    ## Get all the news from the most current to the oldest. It's necessary to use the parse_date_files.py file
    for the news contained in noticias.json being splitted into files by date. ##
    
    $ scrapy crawl noticias --nolog -s filename=noticias.json
    ------------------------------------------------------------------------------------------------------------
    ## Get all the news from the most current to a specific year-month date. ##
    
    $ scrapy crawl noticias --nolog -s filename=2018-09.json -a year=2018 -a month=9
"""

import scrapy, re, time, cfscrape
from proceso.items import NoticiasItem
from datetime import datetime, date, timedelta, tzinfo
from scrapy_splash import SplashRequest, SplashFormRequest


TAG_RE = re.compile(r'<[^>]+>')
def remove_tags(text):
    return TAG_RE.sub('', text)



class UTC(tzinfo):
    """
    Class for Time Zone
    """
    def utcoffset(self, dt):
        ## Time zone for CDMX: UTC-6 ##
        return timedelta(hours=-6)

    def tzname(self, dt):
        ## Time zone name ##
        return 'UTC-6'



class QuotesSpider(scrapy.Spider):
    """
    Basic Scrapy Spider class
    """
    # name = "noticias"
    
    def start_requests(self):
        self.tz = UTC()
        year = getattr(self, 'year', None)
        month = getattr(self, 'month', None)

        self.month_parser = {
            'enero'      : '01', 'febrero' : '02',  'marzo'     : '03',  'abril'     : '04',
            'mayo'       : '05', 'junio'   : '06',  'julio'     : '07',  'agosto'    : '08',
            'septiembre' : '09', 'octubre' : '10',  'noviembre' : '11',  'diciembre' : '12'
        }

        self.baseURL = "https://hemeroteca.proceso.com.mx/"
        login_url = "https://hemeroteca.proceso.com.mx/wp-login.php"

        if year is not None and month is not None:
            self.stop_date = date(int(year), int(month), 22)
            # yield scrapy.Request(url=self.baseURL, callback=self.parse_with_stop_date)
        
        else:
            self.stop_date = None
            # yield scrapy.Request(url=self.baseURL, callback=self.parse)

        # yield scrapy.Request(url=login_url, callback=self.parse_login)
        yield SplashRequest(url=login_url, callback=self.parse_login, endpoint='render.html', args={ 'wait': 0.5 })



    def parse_login(self, response):
        print "parse_login"
        # return scrapy.FormRequest.from_response(
        #     response,
        #     formdata = {
        #         'log' : 'carlos_silvaforne@yahoo.com.mx',
        #         'pwd' : 'carlos_silvaforne@'
        #     },
        #     callback = self.after_login
        # )
        return SplashFormRequest.from_response(
            response,
            formdata = {
                'log' : 'carlos_silvaforne@yahoo.com.mx',
                'pwd' : 'carlos_silvaforne@'
                # 'log' : 'myusr',
                # 'pwd' : 'mypwd'
            },
            callback = self.after_login,
            # callback = self.parse_with_stop_date,
            dont_click = True
        )



    def after_login(self, response):
        ## Check login succeed before going on ##
        print "after_login"
        # print response.body
        if "authentication failed" in response.body:
            self.logger.error("Login failed.")
            return

        else:
            print "passed"
            # token, agent = cfscrape.get_tokens(self.baseURL)
            
            if self.stop_date is None:
                pass
            #     yield scrapy.Request(url=self.baseURL, callback=self.parse, dont_filter=True)

            else:
                # yield scrapy.Request(url=self.baseURL, callback=self.parse_with_stop_date, dont_filter=True)
                # yield SplashRequest(url=self.baseURL, callback=self.parse_with_stop_date)
                yield scrapy.Request(
                    url=self.baseURL,
                    callback=self.parse_with_stop_date,
                    cookies=token,
                    headers={'User-Agent' : agent}
                )


    def parse_with_stop_date(self, response):
        TO_NEXT_PAGE = True

        for item in response.css('div.catpor-box > div'):
            item_date = item.css('span.catpor-published').extract_first()

            if item_date is not None:
                item_date    = remove_tags(item_date).replace(",", '')
                item_date    = item_date.split(' ')
                item_date[1] = self.month_parser[item_date[1]]
                item_date    = map(int, item_date)
                item_date    = date(item_date[2], item_date[1], item_date[0])
                
                if item_date >= self.stop_date:
                    item_link = item.css('span.catpor-title > a::attr(href)').extract_first()
                    yield scrapy.Request(url=item_link, callback=self.parse_links)

                else:
                    TO_NEXT_PAGE = False
                    break

        if TO_NEXT_PAGE:
            next_page = response.css('div.page-navigation > div.nav-next > a::attr(href)').extract_first()

            if next_page is not None:
                yield scrapy.Request(url=next_page, callback=self.parse_with_stop_date)



    def parse_links(self, response):
        for link in response.css('div.post-container > h2 > a::attr(href)').extract():
            # print link
            yield SplashRequest(url=link, callback=self.parse_item, endpoint='render.html', args={ 'wait': 0.5 })


    def parse_item(self, response):
        item = NoticiasItem()
        text = ''
        
        news_date = response.xpath('//div[@id="primary"]').css('span.published').extract_first()
        if news_date is not None:
            news_date = remove_tags(news_date)
            print news_date
            d, t = news_date.split(' ')
            d = map(int, d.split("-"))
            t = map(int, t.split(":"))
            news_date = datetime(d[0], d[1], d[2], t[0], t[1], t[2], tzinfo=self.tz).isoformat('T')

        title = response.xpath('//div[@id="primary"]/div/h1').extract_first()
        if title is not None : title = remove_tags(title)

        topic = response.css('span.entry-categories').extract_first()
        if topic is not None : topic = remove_tags(topic)
        
        for paragraph in response.xpath('//div[@id="primary"]').css('div.entry-content > div').css('p').extract():
            text += remove_tags(paragraph) + '\n'
        
        ## News item info ##
        item['date']  = news_date
        item['title'] = title
        item['topic'] = topic
        item['text']  = text.strip()
        item['url']   = response.url
        
        yield item

