# -*- coding: utf-8 -*-

"""
MEDIA:
    El Financiero, CDMX

USAGE:
    ## Get the news from RSS. ##
    ---------------------------------------------------------------------------------------------
    $ cd elFinanciero/
    $ scrapy crawl noticias --nolog -s filename=2017-12-20.json
"""

import scrapy, re, json
from elFinanciero.items import NoticiasItem
from datetime import datetime, timedelta, tzinfo


TAG_RE = re.compile(r'<[^>]+>')
def remove_tags(text):
    return TAG_RE.sub('', text)


class UTC(tzinfo):
    """
    Class for Time Zone
    """
    def utcoffset(self, dt):
        ## Time zone for CDMX: UTC-6 ##
        return timedelta(hours=-6)

    def tzname(self, dt):
        ## Time zone name ##
        return 'UTC-6'


class QuotesSpider(scrapy.Spider):
    """
    Basic Scrapy Spider class
    """
    name = "noticias"


    def start_requests(self):
        self.tz = UTC()
        self.baseURL = "http://www.elfinanciero.com.mx/rss"

        yield scrapy.Request(url=self.baseURL, callback=self.parse)



    def parse(self, response):
        for link in response.xpath('//link/text()').extract()[1:]:
            yield scrapy.Request(url=link, callback=self.parse_item)



    def parse_item(self, response):
        item = NoticiasItem()
        text = ''

        res = response.xpath('//script[@data-schema="NewsArticle"]').extract_first()
        if res is not None : res = remove_tags(res)
        resDict = json.loads(res)
        dt = resDict['datePublished']
        d,t = dt.split()
        d = map(int, d.split("-"))
        t = map(int, t.split(":"))
        news_date = datetime(d[0], d[1], d[2], t[0], t[1], t[2], tzinfo=self.tz).isoformat("T")

        title = remove_tags(response.css('div.column').css('div.column').css('h1').extract_first()).strip()

        topic = response.xpath('//div[@class="section-line"]').extract_first()
        if topic is not None:
            topic = remove_tags(topic)

        author = response.xpath('//div[@class="note-author"]/a').extract_first()
        if author is not None:
            author = remove_tags(author)

        for p in response.css('div.content').css('p').extract():
            text += remove_tags(p) + '\n'


        ## News item info ##
        item['date']  = news_date
        item['title'] = title
        item['topic'] = topic
        item['author'] = author
        item['text']  = text.strip()
        item['url']  = response.url

        yield item

